@@ -17,7 +17,7 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- * (C) Copyright 2008 - 2010 Red Hat, Inc.
+ * (C) Copyright 2008 - 2011 Red Hat, Inc.
  * (C) Copyright 2008 Novell, Inc.
  */
 
@@ -34,8 +34,9 @@
 #include <nm-setting-gsm.h>
 #include <nm-setting-serial.h>
 #include <nm-setting-ppp.h>
-#include <nm-gsm-device.h>
+#include <nm-device-modem.h>
 #include <nm-utils.h>
+#include <nm-secret-agent.h>
 
 #include "applet.h"
 #include "applet-device-gsm.h"
@@ -64,6 +65,7 @@ typedef struct {
 	NMApplet *applet;
 	NMDevice *device;
 
+	DBusGConnection *bus;
 	DBusGProxy *props_proxy;
 	DBusGProxy *card_proxy;
 	DBusGProxy *net_proxy;
@@ -129,7 +131,7 @@ mobile_wizard_done (MobileWizard *wizard,
 		NMSetting *setting;
 		char *uuid, *id;
 
-		if (method->devtype != NM_DEVICE_TYPE_GSM) {
+		if (method->devtype != NM_DEVICE_MODEM_CAPABILITY_GSM_UMTS) {
 			g_warning ("Unexpected device type (not GSM).");
 			canceled = TRUE;
 			goto done;
@@ -197,7 +199,7 @@ gsm_new_auto_connection (NMDevice *device,
 	info->callback = callback;
 	info->callback_data = callback_data;
 
-	wizard = mobile_wizard_new (NULL, NULL, NM_DEVICE_TYPE_GSM, FALSE,
+	wizard = mobile_wizard_new (NULL, NULL, NM_DEVICE_MODEM_CAPABILITY_GSM_UMTS, FALSE,
 	                            mobile_wizard_done, info);
 	if (wizard) {
 		mobile_wizard_present (wizard);
@@ -206,7 +208,7 @@ gsm_new_auto_connection (NMDevice *device,
 
 	/* Fall back to something */
 	method = g_malloc0 (sizeof (MobileWizardAccessMethod));
-	method->devtype = NM_DEVICE_TYPE_GSM;
+	method->devtype = NM_DEVICE_MODEM_CAPABILITY_GSM_UMTS;
 	method->provider_name = _("GSM");
 	mobile_wizard_done (NULL, FALSE, method, info);
 	g_free (method);
@@ -343,11 +345,12 @@ gsm_add_menu_item (NMDevice *device,
 		item = nm_mb_menu_item_new (nm_setting_connection_get_id (s_con),
 		                            info->quality_valid ? info->quality : 0,
 		                            info->op_name,
+		                            TRUE,
 		                            gsm_act_to_mb_act (info),
 		                            gsm_state_to_mb_state (info),
 		                            info->modem_enabled,
 		                            applet);
-
+		gtk_widget_set_sensitive (GTK_WIDGET (item), TRUE);
 		add_connection_item (device, active, item, menu, applet);
 	}
 
@@ -359,13 +362,16 @@ gsm_add_menu_item (NMDevice *device,
 			gtk_widget_show (item);
 		}
 	} else {
+		/* Otherwise show idle registration state or disabled */
 		item = nm_mb_menu_item_new (NULL,
 		                            info->quality_valid ? info->quality : 0,
 		                            info->op_name,
+		                            FALSE,
 		                            gsm_act_to_mb_act (info),
 		                            gsm_state_to_mb_state (info),
 		                            info->modem_enabled,
 		                            applet);
+		gtk_widget_set_sensitive (GTK_WIDGET (item), FALSE);
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 	}
 
@@ -491,134 +497,24 @@ gsm_get_icon (NMDevice *device,
 }
 
 typedef struct {
-	/* General stuff */
+	SecretsRequest req;
+
 	GtkWidget *dialog;
 	GtkEntry *secret_entry;
 	char *secret_name;
-	NMApplet *applet;
-	NMANewSecretsRequestedFunc callback;
-	gpointer callback_data;
-	NMActiveConnection *active_connection;
-	NMSettingsConnectionInterface *connection;
 } NMGsmSecretsInfo;
 
-
 static void
-secrets_dialog_destroy (gpointer user_data, GObject *finalized)
+free_gsm_secrets_info (SecretsRequest *req)
 {
-	NMGsmSecretsInfo *info = user_data;
+	NMGsmSecretsInfo *info = (NMGsmSecretsInfo *) req;
 
-	gtk_widget_hide (info->dialog);
-	gtk_widget_destroy (info->dialog);
+	if (info->dialog) {
+		gtk_widget_hide (info->dialog);
+		gtk_widget_destroy (info->dialog);
+	}
 
-	g_object_unref (info->connection);
 	g_free (info->secret_name);
-	g_free (info);
-}
-
-static void
-update_cb (NMSettingsConnectionInterface *connection,
-           GError *error,
-           gpointer user_data)
-{
-	if (error) {
-		g_warning ("%s: failed to update connection: (%d) %s",
-		           __func__, error->code, error->message);
-	}
-}
-
-static void
-get_existing_secrets_cb (NMSettingsConnectionInterface *connection,
-                         GHashTable *existing_secrets,
-                         GError *secrets_error,
-                         gpointer user_data)
-{
-	NMGsmSecretsInfo *info = (NMGsmSecretsInfo *) user_data;
-	GHashTable *settings;
-	GError *error = NULL;
-	gboolean save_secret = FALSE;
-	const char *new_secret = NULL;
-
-	if (secrets_error) {
-		error = g_error_copy (secrets_error);
-		goto done;
-	}
-
-	/* Be a bit paranoid */
-	if (connection != info->connection) {
-		g_set_error (&error,
-		             NM_SETTINGS_INTERFACE_ERROR,
-		             NM_SETTINGS_INTERFACE_ERROR_INTERNAL_ERROR,
-		             "%s.%d (%s): unexpected reply for wrong connection.",
-		             __FILE__, __LINE__, __func__);
-		goto done;
-	}
-
-	/* Update connection's secrets so we can hash them for sending back to
-	 * NM, and so we can save them all back out to GConf if needed.
-	 */
-	if (existing_secrets) {
-		GHashTableIter iter;
-		gpointer key, value;
-
-		g_hash_table_iter_init (&iter, existing_secrets);
-		while (g_hash_table_iter_next (&iter, &key, &value)) {
-			GError *update_error = NULL;
-			const char *setting_name = key;
-			GHashTable *setting_hash = value;
-
-			/* Keep track of whether or not the user originall saved the secret */
-			if (!strcmp (setting_name, NM_SETTING_GSM_SETTING_NAME)) {
-				if (g_hash_table_lookup (setting_hash, info->secret_name))
-					save_secret = TRUE;
-			}
-
-			if (!nm_connection_update_secrets (NM_CONNECTION (info->connection),
-			                                   setting_name,
-			                                   setting_hash,
-			                                   &update_error)) {
-				g_warning ("%s: error updating connection secrets: (%d) %s",
-				           __func__,
-				           update_error ? update_error->code : -1,
-				           update_error && update_error->message ? update_error->message : "(unknown)");
-				g_clear_error (&update_error);
-			}
-		}
-	}
-
-	/* Now update the secret the user just entered */
-	if (   !strcmp (info->secret_name, NM_SETTING_GSM_PIN)
-	    || !strcmp (info->secret_name, NM_SETTING_GSM_PASSWORD)) {
-		NMSetting *s_gsm;
-
-		s_gsm = (NMSetting *) nm_connection_get_setting (NM_CONNECTION (info->connection),
-		                                                 NM_TYPE_SETTING_GSM);
-		if (s_gsm) {
-			new_secret = gtk_entry_get_text (info->secret_entry);
-			g_object_set (G_OBJECT (s_gsm), info->secret_name, new_secret, NULL);
-		}
-	}
-
-	settings = nm_connection_to_hash (NM_CONNECTION (info->connection));
-	info->callback (info->connection, settings, NULL, info->callback_data);
-	g_hash_table_destroy (settings);
-
-	/* Save secrets back to GConf if the user had entered them into the
-	 * connection originally.  This lets users enter their secret every time if
-	 * they want.
-	 */
-	if (new_secret && save_secret)
-		nm_settings_connection_interface_update (info->connection, update_cb, NULL);
-
- done:
-	if (error) {
-		g_warning ("%s", error->message);
-		info->callback (info->connection, NULL, error, info->callback_data);
-		g_clear_error (&error);
-	}
-
-	nm_connection_clear_secrets (NM_CONNECTION (info->connection));
-	secrets_dialog_destroy (info, NULL);
 }
 
 static void
@@ -626,46 +522,33 @@ get_gsm_secrets_cb (GtkDialog *dialog,
                     gint response,
                     gpointer user_data)
 {
-	NMGsmSecretsInfo *info = (NMGsmSecretsInfo *) user_data;
+	SecretsRequest *req = user_data;
+	NMGsmSecretsInfo *info = (NMGsmSecretsInfo *) req;
+	NMSetting *setting;
 	GError *error = NULL;
 
-	/* Got a user response, clear the NMActiveConnection destroy handler for
-	 * this dialog since this function will now take over dialog destruction.
-	 */
-	g_object_weak_unref (G_OBJECT (info->active_connection), secrets_dialog_destroy, info);
-
 	if (response == GTK_RESPONSE_OK) {
-		const char *hints[2] = { info->secret_name, NULL };
-
-		/* Get existing connection secrets since NM will want those too */
-		if (!nm_settings_connection_interface_get_secrets (info->connection,
-		                                                   NM_SETTING_GSM_SETTING_NAME,
-		                                                   (const char **) &hints,
-		                                                   FALSE,
-		                                                   get_existing_secrets_cb,
-		                                                   info)) {
-			g_set_error (&error,
-			             NM_SETTINGS_INTERFACE_ERROR,
-			             NM_SETTINGS_INTERFACE_ERROR_INTERNAL_ERROR,
-			             "%s.%d (%s): failed to get existing connection secrets",
-			             __FILE__, __LINE__, __func__);
+		setting = nm_connection_get_setting (req->connection, NM_TYPE_SETTING_GSM);
+		if (setting) {
+			g_object_set (G_OBJECT (setting),
+				          info->secret_name, gtk_entry_get_text (info->secret_entry),
+				          NULL);
+		} else {
+			error = g_error_new (NM_SECRET_AGENT_ERROR,
+				                 NM_SECRET_AGENT_ERROR_INTERNAL_ERROR,
+				                 "%s.%d (%s): no GSM setting",
+				                 __FILE__, __LINE__, __func__);
 		}
 	} else {
-		g_set_error (&error,
-		             NM_SETTINGS_INTERFACE_ERROR,
-		             NM_SETTINGS_INTERFACE_ERROR_INTERNAL_ERROR,
-		             "%s.%d (%s): canceled",
-		             __FILE__, __LINE__, __func__);
+		error = g_error_new (NM_SECRET_AGENT_ERROR,
+		                     NM_SECRET_AGENT_ERROR_USER_CANCELED,
+		                     "%s.%d (%s): canceled",
+		                     __FILE__, __LINE__, __func__);
 	}
 
-	if (error) {
-		g_warning ("%s", error->message);
-		info->callback (info->connection, NULL, error, info->callback_data);
-		g_error_free (error);
-
-		nm_connection_clear_secrets (NM_CONNECTION (info->connection));
-		secrets_dialog_destroy (info, NULL);
-	}
+	applet_secrets_request_complete_setting (req, NM_SETTING_GSM_SETTING_NAME, error);
+	applet_secrets_request_free (req);
+	g_clear_error (&error);
 }
 
 static void
@@ -695,12 +578,11 @@ pin_entry_changed (GtkEditable *editable, gpointer user_data)
 }
 
 static GtkWidget *
-ask_for_pin (NMDevice *device, GtkEntry **out_secret_entry)
+ask_for_pin (GtkEntry **out_secret_entry)
 {
 	GtkDialog *dialog;
 	GtkWidget *w = NULL, *ok_button = NULL;
 	GtkBox *box = NULL, *vbox = NULL;
-	char *dev_str;
 
 	dialog = GTK_DIALOG (gtk_dialog_new ());
 	gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
@@ -715,12 +597,6 @@ ask_for_pin (NMDevice *device, GtkEntry **out_secret_entry)
 	w = gtk_label_new (_("PIN code is needed for the mobile broadband device"));
 	gtk_box_pack_start (vbox, w, TRUE, TRUE, 0);
 
-	dev_str = g_strdup_printf ("<b>%s</b>", utils_get_device_description (device));
-	w = gtk_label_new (NULL);
-	gtk_label_set_markup (GTK_LABEL (w), dev_str);
-	g_free (dev_str);
-	gtk_box_pack_start (vbox, w, TRUE, TRUE, 0);
-
 	w = gtk_alignment_new (0.5, 0.5, 0, 1.0);
 	gtk_box_pack_start (vbox, w, TRUE, TRUE, 0);
 
@@ -745,74 +621,69 @@ ask_for_pin (NMDevice *device, GtkEntry **out_secret_entry)
 }
 
 static gboolean
-gsm_get_secrets (NMDevice *device,
-                 NMSettingsConnectionInterface *connection,
-                 NMActiveConnection *active_connection,
-                 const char *setting_name,
-                 const char **hints,
-                 NMANewSecretsRequestedFunc callback,
-                 gpointer callback_data,
-                 NMApplet *applet,
-                 GError **error)
+gsm_get_secrets (SecretsRequest *req, GError **error)
 {
-	NMGsmSecretsInfo *secrets_info;
+	NMGsmSecretsInfo *info = (NMGsmSecretsInfo *) req;
 	GtkWidget *widget;
 	GtkEntry *secret_entry = NULL;
 
-	if (!hints || !g_strv_length ((char **) hints)) {
+	applet_secrets_request_set_free_func (req, free_gsm_secrets_info);
+
+	if (!req->hints || !g_strv_length (req->hints)) {
 		g_set_error (error,
-		             NM_SETTINGS_INTERFACE_ERROR,
-		             NM_SETTINGS_INTERFACE_ERROR_INTERNAL_ERROR,
+		             NM_SECRET_AGENT_ERROR,
+		             NM_SECRET_AGENT_ERROR_INTERNAL_ERROR,
 		             "%s.%d (%s): missing secrets hints.",
 		             __FILE__, __LINE__, __func__);
 		return FALSE;
 	}
+	info->secret_name = g_strdup (req->hints[0]);
 
-	if (!strcmp (hints[0], NM_SETTING_GSM_PIN)) {
-		GsmDeviceInfo *info = g_object_get_data (G_OBJECT (device), "devinfo");
+	if (!strcmp (info->secret_name, NM_SETTING_GSM_PIN)) {
+		NMDevice *device;
+		GsmDeviceInfo *devinfo;
+
+		device = applet_get_device_for_connection (req->applet, req->connection);
+		if (!device) {
+			g_set_error (error,
+				         NM_SECRET_AGENT_ERROR,
+				         NM_SECRET_AGENT_ERROR_INTERNAL_ERROR,
+				         "%s.%d (%s): failed to find device for active connection.",
+				         __FILE__, __LINE__, __func__);
+			return FALSE;
+		}
+
+		devinfo = g_object_get_data (G_OBJECT (device), "devinfo");
+		g_assert (devinfo);
 
-		g_assert (info);
 		/* A GetSecrets PIN dialog overrides the initial unlock dialog */
-		if (info->dialog)
-			unlock_dialog_destroy (info);
+		if (devinfo->dialog)
+			unlock_dialog_destroy (devinfo);
 
-		widget = ask_for_pin (device, &secret_entry);
-	} else if (!strcmp (hints[0], NM_SETTING_GSM_PASSWORD))
-		widget = applet_mobile_password_dialog_new (device, NM_CONNECTION (connection), &secret_entry);
+		widget = ask_for_pin (&secret_entry);
+	} else if (!strcmp (info->secret_name, NM_SETTING_GSM_PASSWORD))
+		widget = applet_mobile_password_dialog_new (req->connection, &secret_entry);
 	else {
 		g_set_error (error,
-		             NM_SETTINGS_INTERFACE_ERROR,
-		             NM_SETTINGS_INTERFACE_ERROR_INTERNAL_ERROR,
+		             NM_SECRET_AGENT_ERROR,
+		             NM_SECRET_AGENT_ERROR_INTERNAL_ERROR,
 		             "%s.%d (%s): unknown secrets hint '%s'.",
-		             __FILE__, __LINE__, __func__, hints[0]);
+		             __FILE__, __LINE__, __func__, info->secret_name);
 		return FALSE;
 	}
+	info->dialog = widget;
+	info->secret_entry = secret_entry;
 
 	if (!widget || !secret_entry) {
 		g_set_error (error,
-		             NM_SETTINGS_INTERFACE_ERROR,
-		             NM_SETTINGS_INTERFACE_ERROR_INTERNAL_ERROR,
+		             NM_SECRET_AGENT_ERROR,
+		             NM_SECRET_AGENT_ERROR_INTERNAL_ERROR,
 		             "%s.%d (%s): error asking for GSM secrets.",
 		             __FILE__, __LINE__, __func__);
 		return FALSE;
 	}
 
-	secrets_info = g_malloc0 (sizeof (NMGsmSecretsInfo));
-	secrets_info->callback = callback;
-	secrets_info->callback_data = callback_data;
-	secrets_info->applet = applet;
-	secrets_info->active_connection = active_connection;
-	secrets_info->connection = g_object_ref (connection);
-	secrets_info->secret_name = g_strdup (hints[0]);
-	secrets_info->dialog = widget;
-	secrets_info->secret_entry = secret_entry;
-
-	g_signal_connect (widget, "response", G_CALLBACK (get_gsm_secrets_cb), secrets_info);
-
-	/* Attach a destroy notifier to the NMActiveConnection so we can destroy
-	 * the dialog when the active connection goes away.
-	 */
-	g_object_weak_ref (G_OBJECT (active_connection), secrets_dialog_destroy, secrets_info);
+	g_signal_connect (widget, "response", G_CALLBACK (get_gsm_secrets_cb), info);
 
 	gtk_window_set_position (GTK_WINDOW (widget), GTK_WIN_POS_CENTER_ALWAYS);
 	gtk_widget_realize (GTK_WIDGET (widget));
@@ -1022,6 +893,8 @@ gsm_device_info_free (gpointer data)
 		g_object_unref (info->card_proxy);
 	if (info->net_proxy)
 		g_object_unref (info->net_proxy);
+	if (info->bus)
+		dbus_g_connection_unref (info->bus);
 
 	if (info->providers)
 		g_hash_table_destroy (info->providers);
@@ -1106,7 +979,7 @@ find_provider_for_mcc_mnc (GHashTable *table, const char *mccmnc)
 
 	if (name3)
 		return g_strdup (name3);
-	return name2 ? g_strdup (name2) : NULL;
+	return g_strdup (name2);
 }
 
 static char *
@@ -1416,21 +1289,29 @@ modem_properties_changed (DBusGProxy *proxy,
 static void
 gsm_device_added (NMDevice *device, NMApplet *applet)
 {
-	NMGsmDevice *gsm = NM_GSM_DEVICE (device);
-	AppletDBusManager *dbus_mgr = applet_dbus_manager_get ();
-	DBusGConnection *bus = applet_dbus_manager_get_connection (dbus_mgr);
+	NMDeviceModem *modem = NM_DEVICE_MODEM (device);
 	GsmDeviceInfo *info;
 	const char *udi;
+	DBusGConnection *bus;
+	GError *error = NULL;
 
 	udi = nm_device_get_udi (device);
 	if (!udi)
 		return;
 
+	bus = dbus_g_bus_get (DBUS_BUS_SYSTEM, &error);
+	if (!bus) {
+		g_warning ("%s: failed to connect to D-Bus: (%d) %s", __func__, error->code, error->message);
+		g_clear_error (&error);
+		return;
+	}
+
 	info = g_malloc0 (sizeof (GsmDeviceInfo));
 	info->applet = applet;
 	info->device = device;
+	info->bus = bus;
 
-	info->props_proxy = dbus_g_proxy_new_for_name (bus,
+	info->props_proxy = dbus_g_proxy_new_for_name (info->bus,
 	                                               "org.freedesktop.ModemManager",
 	                                               udi,
 	                                               "org.freedesktop.DBus.Properties");
@@ -1440,7 +1321,7 @@ gsm_device_added (NMDevice *device, NMApplet *applet)
 		return;
 	}
 
-	info->card_proxy = dbus_g_proxy_new_for_name (bus,
+	info->card_proxy = dbus_g_proxy_new_for_name (info->bus,
 	                                              "org.freedesktop.ModemManager",
 	                                              udi,
 	                                              "org.freedesktop.ModemManager.Modem.Gsm.Card");
@@ -1450,7 +1331,7 @@ gsm_device_added (NMDevice *device, NMApplet *applet)
 		return;
 	}
 
-	info->net_proxy = dbus_g_proxy_new_for_name (bus,
+	info->net_proxy = dbus_g_proxy_new_for_name (info->bus,
 	                                             "org.freedesktop.ModemManager",
 	                                             udi,
 	                                             MM_DBUS_INTERFACE_MODEM_GSM_NETWORK);
@@ -1460,7 +1341,7 @@ gsm_device_added (NMDevice *device, NMApplet *applet)
 		return;
 	}
 
-	g_object_set_data_full (G_OBJECT (gsm), "devinfo", info, gsm_device_info_free);
+	g_object_set_data_full (G_OBJECT (modem), "devinfo", info, gsm_device_info_free);
 
 	/* Registration info signal */
 	dbus_g_object_register_marshaller (nma_marshal_VOID__UINT_STRING_STRING,
@@ -1507,8 +1388,6 @@ gsm_device_added (NMDevice *device, NMApplet *applet)
 	                         G_TYPE_STRING, MM_DBUS_INTERFACE_MODEM_GSM_NETWORK,
 	                         G_TYPE_STRING, "AccessTechnology",
 	                         G_TYPE_INVALID);
-
-	g_object_unref (dbus_mgr);
 }
 
 NMADeviceClass *
@@ -1525,6 +1404,7 @@ applet_device_gsm_get_class (NMApplet *applet)
 	dclass->device_state_changed = gsm_device_state_changed;
 	dclass->get_icon = gsm_get_icon;
 	dclass->get_secrets = gsm_get_secrets;
+	dclass->secrets_request_size = sizeof (NMGsmSecretsInfo);
 	dclass->device_added = gsm_device_added;
 
 	return dclass;
