@@ -1897,6 +1897,8 @@ nm_gconf_migrate_0_7_ca_cert_ignore (GConfClient *client)
 		                          &ignore_ca_cert);
 		if (ignore_ca_cert)
 			_set_ignore_ca_cert (client, uuid, FALSE);
+		g_free (uuid);
+
 		/* delete old key */
 		unset_one_setting_property (client, dir,
 		                            NM_SETTING_802_1X_SETTING_NAME,
@@ -2035,11 +2037,113 @@ vpnc_type_to_flag (const char *vpnc_type)
 	return NM_SETTING_SECRET_FLAG_NONE;
 }
 
+#define NM_DBUS_SERVICE_VPNC "org.freedesktop.NetworkManager.vpnc"
 #define NM_VPNC_KEY_SECRET "IPSec secret"
 #define NM_VPNC_KEY_SECRET_TYPE "ipsec-secret-type"
 #define NM_VPNC_KEY_XAUTH_PASSWORD "Xauth password"
 #define NM_VPNC_KEY_XAUTH_PASSWORD_TYPE "xauth-password-type"
 
+static void
+migrate_vpnc (NMConnection *connection, NMSettingVPN *s_vpn)
+{
+	NMSettingSecretFlags flags;
+	const char *tmp;
+
+	tmp = nm_setting_vpn_get_data_item (s_vpn, NM_VPNC_KEY_SECRET_TYPE);
+	if (tmp) {
+		flags = vpnc_type_to_flag (tmp) | NM_SETTING_SECRET_FLAG_AGENT_OWNED;
+		nm_setting_set_secret_flags (NM_SETTING (s_vpn), NM_VPNC_KEY_SECRET, flags, NULL);
+	}
+
+	tmp = nm_setting_vpn_get_data_item (s_vpn, NM_VPNC_KEY_XAUTH_PASSWORD_TYPE);
+	if (tmp) {
+		flags = vpnc_type_to_flag (tmp) | NM_SETTING_SECRET_FLAG_AGENT_OWNED;
+		nm_setting_set_secret_flags (NM_SETTING (s_vpn), NM_VPNC_KEY_XAUTH_PASSWORD, flags, NULL);
+	}
+}
+
+#define NM_DBUS_SERVICE_OPENVPN "org.freedesktop.NetworkManager.openvpn"
+#define NM_OPENVPN_KEY_PASSWORD "password"
+#define NM_OPENVPN_KEY_CERTPASS "cert-pass"
+#define NM_OPENVPN_KEY_HTTP_PROXY_PASSWORD "http-proxy-password"
+#define NM_OPENVPN_KEY_CONNECTION_TYPE "connection-type"
+#define NM_OPENVPN_CONTYPE_TLS          "tls"
+#define NM_OPENVPN_CONTYPE_PASSWORD     "password"
+#define NM_OPENVPN_CONTYPE_PASSWORD_TLS "password-tls"
+#define NM_OPENVPN_KEY_PROXY_TYPE "proxy-type"
+
+static NMSettingSecretFlags
+openvpn_get_secret_flags (const char *uuid, const char *secret_name)
+{
+	GList *found = NULL;
+	GnomeKeyringResult ret;
+	NMSettingSecretFlags flags = NM_SETTING_SECRET_FLAG_AGENT_OWNED;
+
+	ret = gnome_keyring_find_itemsv_sync (GNOME_KEYRING_ITEM_GENERIC_SECRET,
+		                                  &found,
+		                                  KEYRING_UUID_TAG,
+		                                  GNOME_KEYRING_ATTRIBUTE_TYPE_STRING,
+										  uuid,
+		                                  KEYRING_SN_TAG,
+		                                  GNOME_KEYRING_ATTRIBUTE_TYPE_STRING,
+		                                  NM_SETTING_VPN_SETTING_NAME,
+		                                  KEYRING_SK_TAG,
+		                                  GNOME_KEYRING_ATTRIBUTE_TYPE_STRING,
+		                                  secret_name,
+		                                  NULL);
+	if (ret != GNOME_KEYRING_RESULT_OK || found == NULL)
+		flags |= NM_SETTING_SECRET_FLAG_NOT_SAVED;
+	gnome_keyring_found_list_free (found);
+
+	return flags;
+}
+
+static void
+migrate_openvpn (NMConnection *connection, NMSettingVPN *s_vpn)
+{
+	NMSettingSecretFlags flags;
+	const char *tmp;
+	gboolean check_pw = FALSE, check_cp = FALSE;
+
+	tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_CONNECTION_TYPE);
+	if (!tmp)
+		return;
+
+	if (!strcmp (tmp, NM_OPENVPN_CONTYPE_TLS))
+		check_cp = TRUE;
+	else if (!strcmp (tmp, NM_OPENVPN_CONTYPE_PASSWORD_TLS)) {
+		check_pw = TRUE;
+		check_cp = TRUE;
+	} else if (!strcmp (tmp, NM_OPENVPN_CONTYPE_PASSWORD))
+		check_pw = TRUE;
+
+	/* For each secret, we need to check the keyring to see whether the secret
+	 * is present or not, and if it's *not*, then we mark the secret as both
+	 * not-saved and agent-owned.  If it is present, the secret is just marked
+	 * agent-owned.
+	 */
+
+	if (check_pw) {
+		flags = openvpn_get_secret_flags (nm_connection_get_uuid (connection), NM_OPENVPN_KEY_PASSWORD);
+		nm_setting_set_secret_flags (NM_SETTING (s_vpn), NM_OPENVPN_KEY_PASSWORD, flags, NULL);
+	}
+
+	if (check_cp) {
+		flags = openvpn_get_secret_flags (nm_connection_get_uuid (connection), NM_OPENVPN_KEY_CERTPASS);
+		nm_setting_set_secret_flags (NM_SETTING (s_vpn), NM_OPENVPN_KEY_CERTPASS, flags, NULL);
+	}
+
+	/* HTTP proxy password */
+	tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_PROXY_TYPE);
+	if (g_strcmp0 (tmp, "http") == 0 || g_strcmp0 (tmp, "socks") == 0) {
+		flags = openvpn_get_secret_flags (nm_connection_get_uuid (connection), NM_OPENVPN_KEY_HTTP_PROXY_PASSWORD);
+		nm_setting_set_secret_flags (NM_SETTING (s_vpn), NM_OPENVPN_KEY_HTTP_PROXY_PASSWORD, flags, NULL);
+	}
+}
+
+#define NM_DBUS_SERVICE_PPTP "org.freedesktop.NetworkManager.pptp"
+#define NM_PPTP_KEY_PASSWORD "password"
+
 void
 nm_gconf_migrate_09_secret_flags (GConfClient *client,
                                   NMConnection *connection,
@@ -2057,24 +2161,27 @@ nm_gconf_migrate_09_secret_flags (GConfClient *client,
 	if (!setting)
 		return;
 
-	/* Migrate vpnc secret flags */
+	/* Migrate various VPN secret flags */
 	if (NM_IS_SETTING_VPN (setting)) {
-		NMSettingSecretFlags flags;
 		NMSettingVPN *s_vpn = NM_SETTING_VPN (setting);
-		const char *tmp;
+		const char *service;
 
-		/* vpnc stuff */
-		tmp = nm_setting_vpn_get_data_item (s_vpn, NM_VPNC_KEY_SECRET_TYPE);
-		if (tmp) {
-			flags = vpnc_type_to_flag (tmp) | NM_SETTING_SECRET_FLAG_AGENT_OWNED;
-			nm_setting_set_secret_flags (setting, NM_VPNC_KEY_SECRET, flags, NULL);
+		service = nm_setting_vpn_get_service_type (s_vpn);
+		if (g_strcmp0 (service, NM_DBUS_SERVICE_VPNC) == 0) {
+			migrate_vpnc (connection, s_vpn);
+			return;
+		} else if (g_strcmp0 (service, NM_DBUS_SERVICE_PPTP) == 0) {
+			/* Mark the password as agent-owned */
+			nm_setting_set_secret_flags (setting, NM_PPTP_KEY_PASSWORD, NM_SETTING_SECRET_FLAG_AGENT_OWNED, NULL);
+			return;
+		} else if (g_strcmp0 (service, NM_DBUS_SERVICE_OPENVPN) == 0) {
+			migrate_openvpn (connection, s_vpn);
+			return;
 		}
 
-		tmp = nm_setting_vpn_get_data_item (s_vpn, NM_VPNC_KEY_XAUTH_PASSWORD_TYPE);
-		if (tmp) {
-			flags = vpnc_type_to_flag (tmp) | NM_SETTING_SECRET_FLAG_AGENT_OWNED;
-			nm_setting_set_secret_flags (setting, NM_VPNC_KEY_XAUTH_PASSWORD, flags, NULL);
-		}
+		/* Other VPNs not handled specially here just go through the
+		 * generic secrets flags stuff below.
+		 */
 	}
 
 	/* 802.1x connections might be 'always-ask' */
